﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Resources;
using System.Text;
using LockCrypt.Core.Properties;

namespace LockCrypt.Core {
    public class I18nUtils {
        private static ResourceManager resStrings = Strings.ResourceManager,
                                       resErrors = Errors.ResourceManager;

        /// <summary>
        /// Gets a localized version of a string.
        /// </summary>
        /// <param name="bundle">The name of the resource bundle.</param>
        /// <param name="key">The translation key.</param>
        /// <returns>A localized version of the string representyed by <c>key</c></returns>
        public static string GetString(string bundle, string key) {
            string translated = string.Empty;
            switch(bundle) {
                case "Strings":
                    translated = resStrings.GetString(key);
                    break;
                case "Errors":
                    translated = resErrors.GetString(key);
                    break;
            }
            if(translated == null) {
                throw new Exception(string.Format("Resource not found: '{0}.{1}'", bundle, key));
            }
            return translated;
        }

        /// <summary>
        /// Gets a localized version of a formatted string.
        /// </summary>
        /// <param name="bundle">The name of the resource bundle.</param>
        /// <param name="key">The translation key.</param>
        /// <returns>A localized version of the string representyed by <c>key</c></returns>
        public static string GetString(string bundle, string key, params string[] formatArguments) {
            string translated = string.Empty;
            switch(bundle) {
                case "Strings":
                    translated = resStrings.GetString(key);
                    break;
                case "Errors":
                    translated = resErrors.GetString(key);
                    break;
            }
            if(translated == null) {
                throw new Exception(string.Format("Resource not found: '{0}.{1}'", bundle, key));
            }
            string formatted = string.Format(translated, formatArguments);
            return formatted;
        }
    }
}
