package com.lime49.lockcrypt.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import javax.swing.*;
import javax.swing.border.*;
//TODO: add copyright------------------------------------------------------------
/**
 * A list of checkboxes
 */
public class CheckBoxList<T> extends JList {
    protected static Border noFocusBorder = new EmptyBorder(1, 4, 1, 4);
    private CheckBoxListModel<T> model;

    /**
     * 
     * @param items
     */
    public CheckBoxList(LinkedHashMap<T, Boolean> items) {
        model = new CheckBoxListModel<T>(items);
        setModel(model);
        addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                int index = locationToIndex(e.getPoint());
                if(index != -1) {
                    ListItem<T> item = (ListItem<T>)model.getElementAt(index);
                    item.selected = !item.selected;
                    revalidate();
                    repaint();
                }
            }
        });
        setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        setCellRenderer(new CellRenderer());
    }

    /**
     *
     */
    public void selectAll() {
        model.selectAll();
        revalidate();
        repaint();
    }

    /**
     *
     */
    public void selectNone() {
        model.selectNone();
        revalidate();
        repaint();
    }

    @Override
    public String getToolTipText(MouseEvent evt) {
        int index = locationToIndex(evt.getPoint());
        if(index == -1) {
            return "";
        } else {
            ListItem<T> item = (ListItem<T>)model.getElementAt(index);
            return item.dataItem.toString();
        }
    }

    /**
     *
     */
    protected class CellRenderer implements ListCellRenderer {
        private JCheckBox checkBox;
        private Box containerBox;
        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                    boolean isSelected, boolean cellHasFocus) {
            if(containerBox == null) {
                containerBox = Box.createHorizontalBox();
                checkBox = new JCheckBox();
                checkBox.setEnabled(isEnabled());
                checkBox.setFont(getFont());
                checkBox.setFocusPainted(false);
                checkBox.setBorderPainted(true);
                checkBox.setBorder(isSelected ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);
                containerBox.add(checkBox);
            }
            checkBox.setBackground(isSelected ? getSelectionBackground() : getBackground());
            checkBox.setForeground(isSelected ? getSelectionForeground() : getForeground());
            if(value != null) {
                ListItem item = (ListItem)value;
                checkBox.setText(item.dataItem.toString());
                checkBox.setSelected(item.selected);
            }
            return containerBox;
        }
    }

    /**
     *
     * @return
     */
    public LinkedHashMap<T, Boolean> getItems() {
        return model.getItems();
    }

    /**
     *
     * @param <T>
     */
    class CheckBoxListModel<T> extends AbstractListModel {
        private LinkedList<ListItem<T>> items = new LinkedList<ListItem<T>>(); // damn Java for lacking an indexOf method in LinkedHashMap

        /**
         *
         * @param items
         */
        public CheckBoxListModel(LinkedHashMap<T, Boolean> items) {
            Iterator<T> iter = items.keySet().iterator();
            while(iter.hasNext()) {
                T acc = iter.next();
                this.items.add(new ListItem<T>(acc, items.get(acc)));
            }
        }

        /**
         *
         * @return
         */
        public int getSize() {
            return items.size();
        }

        /**
         *
         * @param index
         * @return
         */
        public ListItem<T> getElementAt(int index) {
            return items.get(index);
        }

        /**
         *
         * @return
         */
        private LinkedHashMap<T, Boolean> getItems() {
            LinkedHashMap<T, Boolean> map = new LinkedHashMap<T, Boolean>();
            for(ListItem<T> item : items) {
                map.put(item.dataItem, item.selected);
            }
            return map;
        }

        /**
         *
         */
        public void selectAll() {
            for(ListItem<T> item : items) {
                item.selected = true;
            }
        }

        /**
         *
         */
        public void selectNone() {
            for(ListItem<T> item : items) {
                item.selected = false;
            }
        }
    }

    /**
     *
     * @param <T>
     */
    class ListItem<T> {
        public T dataItem;
        public boolean selected;
        
        /**
         *
         * @param dataItem
         * @param selected
         */
        public ListItem(T dataItem, boolean selected) {
            this.dataItem = dataItem;
            this.selected = selected;
        }
    }
}
